/* cmdline.h */

/* File autogenerated by gengetopt version 2.14  */

#ifndef CMDLINE_H
#define CMDLINE_H

/* If we use autoconf.  */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef CMDLINE_PARSER_PACKAGE
#define CMDLINE_PARSER_PACKAGE "ivtv-tune"
#endif

#ifndef CMDLINE_PARSER_VERSION
#define CMDLINE_PARSER_VERSION "1.50"
#endif

struct gengetopt_args_info
{
  char * channel_arg;	/* set new channel.  */
  char * channel_orig;	/* set new channel original value given at command line.  */
  char * device_arg;	/* set video device node.  */
  char * device_orig;	/* set video device node original value given at command line.  */
  float frequency_arg;	/* set new frequency (MHz).  */
  char * frequency_orig;	/* set new frequency (MHz) original value given at command line.  */
  int list_channels_flag;	/* list all channels and their frequencies (default=off).  */
  int list_freqtable_flag;	/* list all available frequency mappings (default=off).  */
  char * freqtable_arg;	/* set frequency map to use.  */
  char * freqtable_orig;	/* set frequency map to use original value given at command line.  */
  char * xawtv_arg;	/* set new channel using custom map from ~/.xawtv.  */
  char * xawtv_orig;	/* set new channel using custom map from ~/.xawtv original value given at command line.  */
  
  int help_given ;	/* Whether help was given.  */
  int version_given ;	/* Whether version was given.  */
  int channel_given ;	/* Whether channel was given.  */
  int device_given ;	/* Whether device was given.  */
  int frequency_given ;	/* Whether frequency was given.  */
  int list_channels_given ;	/* Whether list-channels was given.  */
  int list_freqtable_given ;	/* Whether list-freqtable was given.  */
  int freqtable_given ;	/* Whether freqtable was given.  */
  int xawtv_given ;	/* Whether xawtv was given.  */

} ;

int cmdline_parser (int argc, char * const *argv, struct gengetopt_args_info *args_info);
int cmdline_parser2 (int argc, char * const *argv, struct gengetopt_args_info *args_info, int override, int initialize, int check_required);
int cmdline_parser_file_save(const char *filename, struct gengetopt_args_info *args_info);

void cmdline_parser_print_help(void);
void cmdline_parser_print_version(void);

void cmdline_parser_init (struct gengetopt_args_info *args_info);
void cmdline_parser_free (struct gengetopt_args_info *args_info);

int cmdline_parser_configfile (char * const filename, struct gengetopt_args_info *args_info, int override, int initialize, int check_required);

int cmdline_parser_required (struct gengetopt_args_info *args_info, const char *prog_name);


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* CMDLINE_H */
